@@ -38,20 +38,22 @@ def _cubic_interpolate(x1, f1, g1, x2, f2, g2, bounds=None):
             min_pos = x1 - (x1 - x2) * ((g1 + d2 - d1) / (g1 - g2 + 2 * d2))
         return min(max(min_pos, xmin_bound), xmax_bound)
     else:
-        return (xmin_bound + xmax_bound) / 2.
+        return (xmin_bound + xmax_bound) / 2.0
 
 
-def _strong_wolfe(obj_func,
-                  x,
-                  t,
-                  d,
-                  f,
-                  g,
-                  gtd,
-                  c1=1e-4,
-                  c2=0.9,
-                  tolerance_change=1e-9,
-                  max_ls=25):
+def _strong_wolfe(
+    obj_func,
+    x,
+    t,
+    d,
+    f,
+    g,
+    gtd,
+    c1=1e-4,
+    c2=0.9,
+    tolerance_change=1e-9,
+    max_ls=25,
+):
     # ported from https://github.com/torch/optim/blob/master/lswolfe.lua
     d_norm = d.abs().max()
     g = g.clone(memory_format=torch.contiguous_format)
@@ -59,10 +61,12 @@ def _strong_wolfe(obj_func,
         # evaluate objective and gradient using initial step
         # backtrack until the step stays in the domain
         f_new, g_new = obj_func(x, t, d)
-        if (np.isnan(f_new).any() or
-                np.isinf(f_new).any() or
-                torch.isnan(g_new).any() or
-                torch.isinf(g_new).any()):
+        if (
+            np.isnan(f_new).any()
+            or np.isinf(f_new).any()
+            or torch.isnan(g_new).any()
+            or torch.isinf(g_new).any()
+        ):
             t *= 0.5
         else:
             break
@@ -88,7 +92,8 @@ def _strong_wolfe(obj_func,
             bracket_f = [f_prev, f_new]
             bracket_g = [
                 g_prev,
-                g_new.clone(memory_format=torch.contiguous_format)]
+                g_new.clone(memory_format=torch.contiguous_format),
+            ]
             bracket_gtd = [gtd_prev, gtd_new]
             break
 
@@ -104,7 +109,8 @@ def _strong_wolfe(obj_func,
             bracket_f = [f_prev, f_new]
             bracket_g = [
                 g_prev,
-                g_new.clone(memory_format=torch.contiguous_format)]
+                g_new.clone(memory_format=torch.contiguous_format),
+            ]
             bracket_gtd = [gtd_prev, gtd_new]
             break
 
@@ -119,7 +125,8 @@ def _strong_wolfe(obj_func,
             t,
             f_new,
             gtd_new,
-            bounds=(min_step, max_step))
+            bounds=(min_step, max_step),
+        )
 
         # next step
         t_prev = tmp
@@ -149,8 +156,14 @@ def _strong_wolfe(obj_func,
             break
 
         # compute new trial value
-        t = _cubic_interpolate(bracket[0], bracket_f[0], bracket_gtd[0],
-                               bracket[1], bracket_f[1], bracket_gtd[1])
+        t = _cubic_interpolate(
+            bracket[0],
+            bracket_f[0],
+            bracket_gtd[0],
+            bracket[1],
+            bracket_f[1],
+            bracket_gtd[1],
+        )
 
         # test that we are making sufficient progress:
         # in case `t` is so close to boundary, we mark that we are making
@@ -181,15 +194,18 @@ def _strong_wolfe(obj_func,
         ls_iter += 1
 
         if np.isnan(f_new) or (
-                f_new > (f + c1 * t * gtd) or f_new >= bracket_f[low_pos]):
+            f_new > (f + c1 * t * gtd) or f_new >= bracket_f[low_pos]
+        ):
             # Armijo condition not satisfied or not lower than lowest point
             bracket[high_pos] = t
             bracket_f[high_pos] = f_new
             bracket_g[high_pos] = g_new.clone(
-                memory_format=torch.contiguous_format)
+                memory_format=torch.contiguous_format
+            )
             bracket_gtd[high_pos] = gtd_new
             low_pos, high_pos = (
-                (0, 1) if bracket_f[0] <= bracket_f[1] else (1, 0))
+                (0, 1) if bracket_f[0] <= bracket_f[1] else (1, 0)
+            )
         else:
             if abs(gtd_new) <= -c2 * gtd:
                 # Wolfe conditions satisfied
@@ -205,7 +221,8 @@ def _strong_wolfe(obj_func,
             bracket[low_pos] = t
             bracket_f[low_pos] = f_new
             bracket_g[low_pos] = g_new.clone(
-                memory_format=torch.contiguous_format)
+                memory_format=torch.contiguous_format
+            )
             bracket_gtd[low_pos] = gtd_new
 
     # TODO(akshayka): Fix the line search to handle infinite values / nans.
@@ -218,7 +235,7 @@ def _strong_wolfe(obj_func,
         f_new = bracket_f[low_pos]
         g_new = bracket_g[low_pos]
     except IndexError:
-        t = 1.
+        t = 1.0
         line_search_failed = True
 
     if line_search_failed:
@@ -230,7 +247,7 @@ def _strong_wolfe(obj_func,
             elif f_new < f + c1 * t * gtd:
                 break
     if np.isnan(f_new):
-        t = 0.
+        t = 0.0
         f_new, g_new = obj_func(x, t, d)
 
     return f_new, g_new, t, ls_func_evals
@@ -272,18 +289,20 @@ class LBFGS(torch.optim.Optimizer):
             at the end of the previous iteration.
     """
 
-    def __init__(self,
-                 params,
-                 lr=1,
-                 max_iter=20,
-                 max_eval=None,
-                 tolerance_grad=1e-7,
-                 tolerance_change=1e-9,
-                 history_size=100,
-                 line_search_fn=None,
-                 callback=None,
-                 project_callback=None,
-                 use_cached_loss=False):
+    def __init__(
+        self,
+        params,
+        lr=1,
+        max_iter=20,
+        max_eval=None,
+        tolerance_grad=1e-7,
+        tolerance_change=1e-9,
+        history_size=100,
+        line_search_fn=None,
+        callback=None,
+        project_callback=None,
+        use_cached_loss=False,
+    ):
         if max_eval is None:
             max_eval = max_iter * 5 // 4
         defaults = dict(
@@ -293,14 +312,17 @@ class LBFGS(torch.optim.Optimizer):
             tolerance_grad=tolerance_grad,
             tolerance_change=tolerance_change,
             history_size=history_size,
-            line_search_fn=line_search_fn)
+            line_search_fn=line_search_fn,
+        )
         super(LBFGS, self).__init__(params, defaults)
 
         if len(self.param_groups) != 1:
-            raise ValueError("LBFGS doesn't support per-parameter options "
-                             "(parameter groups)")
+            raise ValueError(
+                "LBFGS doesn't support per-parameter options "
+                "(parameter groups)"
+            )
 
-        self._params = self.param_groups[0]['params']
+        self._params = self.param_groups[0]["params"]
         self._numel_cache = None
         self._callback = callback
         self._project_callback = project_callback
@@ -309,7 +331,8 @@ class LBFGS(torch.optim.Optimizer):
     def _numel(self):
         if self._numel_cache is None:
             self._numel_cache = reduce(
-                lambda total, p: total + p.numel(), self._params, 0)
+                lambda total, p: total + p.numel(), self._params, 0
+            )
         return self._numel_cache
 
     def _gather_flat_grad(self):
@@ -329,14 +352,14 @@ class LBFGS(torch.optim.Optimizer):
         for p in self._params:
             numel = p.numel()
             # view as to avoid deprecated pointwise semantics
-            p.add_(update[offset:offset + numel].view_as(p), alpha=step_size)
+            p.add_(update[offset : offset + numel].view_as(p), alpha=step_size)
             offset += numel
         assert offset == self._numel()
 
     def _clone_param(self):
         return [
-            p.clone(memory_format=torch.contiguous_format)
-            for p in self._params]
+            p.clone(memory_format=torch.contiguous_format) for p in self._params
+        ]
 
     def _set_param(self, params_data):
         for p, pdata in zip(self._params, params_data):
@@ -355,14 +378,14 @@ class LBFGS(torch.optim.Optimizer):
     @torch.no_grad()
     def reset(self):
         state = self.state[self._params[0]]
-        state.pop('d')
-        state.pop('t')
-        state.pop('old_dirs')
-        state.pop('ro')
-        state.pop('H_diag')
-        state.pop('prev_flat_grad')
-        state.pop('prev_loss')
-        state['n_iter'] = 0
+        state.pop("d")
+        state.pop("t")
+        state.pop("old_dirs")
+        state.pop("ro")
+        state.pop("H_diag")
+        state.pop("prev_flat_grad")
+        state.pop("prev_loss")
+        state["n_iter"] = 0
 
     @torch.no_grad()
     def step(self, closure):
@@ -378,23 +401,26 @@ class LBFGS(torch.optim.Optimizer):
         closure = torch.enable_grad()(closure)
 
         group = self.param_groups[0]
-        lr = group['lr']
-        max_iter = group['max_iter']
-        max_eval = group['max_eval']
-        tolerance_grad = group['tolerance_grad']
-        tolerance_change = group['tolerance_change']
-        line_search_fn = group['line_search_fn']
-        history_size = group['history_size']
+        lr = group["lr"]
+        max_iter = group["max_iter"]
+        max_eval = group["max_eval"]
+        tolerance_grad = group["tolerance_grad"]
+        tolerance_change = group["tolerance_change"]
+        line_search_fn = group["line_search_fn"]
+        history_size = group["history_size"]
 
         # NOTE: LBFGS has only global state, but we register it as state for
         # the first param, because this helps with casting in load_state_dict
         state = self.state[self._params[0]]
-        state.setdefault('func_evals', 0)
-        state.setdefault('n_iter', 0)
+        state.setdefault("func_evals", 0)
+        state.setdefault("n_iter", 0)
 
         # evaluate initial f(x) and df/dx
-        if (self._use_cached_loss and state['n_iter'] > 0 and
-                line_search_fn is not None):
+        if (
+            self._use_cached_loss
+            and state["n_iter"] > 0
+            and line_search_fn is not None
+        ):
             orig_loss = self._cached_loss
         else:
             orig_loss = closure()
@@ -403,7 +429,7 @@ class LBFGS(torch.optim.Optimizer):
         loss = float(orig_loss)
 
         current_evals = 1
-        state['func_evals'] += 1
+        state["func_evals"] += 1
 
         flat_grad = self._gather_flat_grad()
         opt_cond = flat_grad.abs().max() <= tolerance_grad
@@ -413,26 +439,26 @@ class LBFGS(torch.optim.Optimizer):
             return orig_loss
 
         # tensors cached in state (for tracing)
-        d = state.get('d')
-        t = state.get('t')
-        old_dirs = state.get('old_dirs')
-        old_stps = state.get('old_stps')
-        ro = state.get('ro')
-        H_diag = state.get('H_diag')
-        prev_flat_grad = state.get('prev_flat_grad')
-        prev_loss = state.get('prev_loss')
+        d = state.get("d")
+        t = state.get("t")
+        old_dirs = state.get("old_dirs")
+        old_stps = state.get("old_stps")
+        ro = state.get("ro")
+        H_diag = state.get("H_diag")
+        prev_flat_grad = state.get("prev_flat_grad")
+        prev_loss = state.get("prev_loss")
 
         n_iter = 0
         # optimize for a max of max_iter iterations
         while n_iter < max_iter:
             # keep track of nb of iterations
             n_iter += 1
-            state['n_iter'] += 1
+            state["n_iter"] += 1
 
             ############################################################
             # compute gradient descent direction
             ############################################################
-            if state['n_iter'] == 1:
+            if state["n_iter"] == 1:
                 d = flat_grad.neg()
                 old_dirs = []
                 old_stps = []
@@ -454,7 +480,7 @@ class LBFGS(torch.optim.Optimizer):
                     # store new direction/step
                     old_dirs.append(y)
                     old_stps.append(s)
-                    ro.append(1. / ys)
+                    ro.append(1.0 / ys)
 
                     # update scale of initial Hessian approximation
                     H_diag = ys / y.dot(y)  # (y*y)
@@ -463,9 +489,9 @@ class LBFGS(torch.optim.Optimizer):
                 # multiplied by the gradient
                 num_old = len(old_dirs)
 
-                if 'al' not in state:
-                    state['al'] = [None] * history_size
-                al = state['al']
+                if "al" not in state:
+                    state["al"] = [None] * history_size
+                al = state["al"]
 
                 # iteration in L-BFGS loop collapsed to use just one buffer
                 q = flat_grad.neg()
@@ -482,7 +508,8 @@ class LBFGS(torch.optim.Optimizer):
 
             if prev_flat_grad is None:
                 prev_flat_grad = flat_grad.clone(
-                    memory_format=torch.contiguous_format)
+                    memory_format=torch.contiguous_format
+                )
             else:
                 prev_flat_grad.copy_(flat_grad)
             prev_loss = loss
@@ -491,8 +518,8 @@ class LBFGS(torch.optim.Optimizer):
             # compute step length
             ############################################################
             # reset initial guess for step size
-            if state['n_iter'] == 1:
-                t = min(1., 1. / flat_grad.abs().sum()) * lr
+            if state["n_iter"] == 1:
+                t = min(1.0, 1.0 / flat_grad.abs().sum()) * lr
             else:
                 t = lr
 
@@ -514,10 +541,12 @@ class LBFGS(torch.optim.Optimizer):
 
                     def obj_func(x, t, d):
                         return self._directional_evaluate(
-                            closure, x, t, d, self._project_callback)
+                            closure, x, t, d, self._project_callback
+                        )
 
                     loss, flat_grad, t, ls_func_evals = _strong_wolfe(
-                        obj_func, x_init, t, d, loss, flat_grad, gtd)
+                        obj_func, x_init, t, d, loss, flat_grad, gtd
+                    )
                 self._cached_loss = torch.tensor(loss, device=flat_grad.device)
                 self._add_grad(t, d)
                 opt_cond = flat_grad.abs().max() <= tolerance_grad
@@ -527,7 +556,7 @@ class LBFGS(torch.optim.Optimizer):
 
             # update func eval
             current_evals += ls_func_evals
-            state['func_evals'] += ls_func_evals
+            state["func_evals"] += ls_func_evals
 
             ############################################################
             # check conditions
@@ -549,13 +578,13 @@ class LBFGS(torch.optim.Optimizer):
             if abs(loss - prev_loss) < tolerance_change:
                 break
 
-        state['d'] = d
-        state['t'] = t
-        state['old_dirs'] = old_dirs
-        state['old_stps'] = old_stps
-        state['ro'] = ro
-        state['H_diag'] = H_diag
-        state['prev_flat_grad'] = prev_flat_grad
-        state['prev_loss'] = prev_loss
+        state["d"] = d
+        state["t"] = t
+        state["old_dirs"] = old_dirs
+        state["old_stps"] = old_stps
+        state["ro"] = ro
+        state["H_diag"] = H_diag
+        state["prev_flat_grad"] = prev_flat_grad
+        state["prev_loss"] = prev_loss
 
         return orig_loss
